@@ -1,20 +1,21 @@
 """Basic tests for the cherrypy.Request object."""
 
+from functools import wraps
 import os
 import sys
 import types
-
-import six
+import uuid
+from http.client import IncompleteRead
 
 import cherrypy
-from cherrypy._cpcompat import IncompleteRead, ntob, ntou
+from cherrypy._cpcompat import ntou
 from cherrypy.lib import httputil
 from cherrypy.test import helper
 
 localDir = os.path.dirname(__file__)
 
 defined_http_methods = ('OPTIONS', 'GET', 'HEAD', 'POST', 'PUT', 'DELETE',
-                        'TRACE', 'PROPFIND')
+                        'TRACE', 'PROPFIND', 'PATCH')
 
 
 #                             Client-side code                             #
@@ -34,6 +35,28 @@ class RequestObjectTests(helper.CPWebCase):
             def scheme(self):
                 return cherrypy.request.scheme
 
+            @cherrypy.expose
+            def created_example_com_3128(self):
+                """Handle CONNECT method."""
+                cherrypy.response.status = 204
+
+            @cherrypy.expose
+            def body_example_com_3128(self):
+                """Handle CONNECT method."""
+                return (
+                    cherrypy.request.method
+                    + 'ed to '
+                    + cherrypy.request.path_info
+                )
+
+            @cherrypy.expose
+            def request_uuid4(self):
+                return [
+                    str(cherrypy.request.unique_id),
+                    ' ',
+                    str(cherrypy.request.unique_id),
+                ]
+
         root = Root()
 
         class TestType(type):
@@ -72,6 +95,12 @@ class RequestObjectTests(helper.CPWebCase):
             def __call__(self):
                 return 'data'
 
+        def handler_dec(f):
+            @wraps(f)
+            def wrapper(handler, *args, **kwargs):
+                return f(handler, *args, **kwargs)
+            return wrapper
+
         class ParamErrors(Test):
 
             @cherrypy.expose
@@ -116,6 +145,11 @@ class RequestObjectTests(helper.CPWebCase):
             def raise_type_error_with_default_param(self, x, y=None):
                 return '%d' % 'a'  # throw an exception
 
+            @cherrypy.expose
+            @handler_dec
+            def raise_type_error_decorated(self, *args, **kwargs):
+                raise TypeError('Client Error')
+
         def callable_error_page(status, **kwargs):
             return "Error %s - Well, I'm very sorry but you haven't paid!" % (
                 status)
@@ -207,7 +241,7 @@ class RequestObjectTests(helper.CPWebCase):
 
             def ifmatch(self):
                 val = cherrypy.request.headers['If-Match']
-                assert isinstance(val, six.text_type)
+                assert isinstance(val, str)
                 cherrypy.response.headers['ETag'] = val
                 return val
 
@@ -215,7 +249,7 @@ class RequestObjectTests(helper.CPWebCase):
 
             def get_elements(self, headername):
                 e = cherrypy.request.headers.elements(headername)
-                return '\n'.join([six.text_type(x) for x in e])
+                return '\n'.join([str(x) for x in e])
 
         class Method(Test):
 
@@ -240,7 +274,7 @@ class RequestObjectTests(helper.CPWebCase):
             def reachable(self):
                 return 'success'
 
-        class Divorce:
+        class Divorce(Test):
 
             """HTTP Method handlers shouldn't collide with normal method names.
             For example, a GET-handler shouldn't collide with a method named
@@ -269,8 +303,6 @@ class RequestObjectTests(helper.CPWebCase):
                 return ('Divorce document %s: %s' %
                         (ID, self.documents.get(ID, 'empty')))
 
-        root.divorce = Divorce()
-
         class ThreadLocal(Test):
 
             def index(self):
@@ -280,7 +312,8 @@ class RequestObjectTests(helper.CPWebCase):
 
         appconf = {
             '/method': {
-                'request.methods_with_bodies': ('POST', 'PUT', 'PROPFIND')
+                'request.methods_with_bodies': ('POST', 'PUT', 'PROPFIND',
+                                                'PATCH')
             },
         }
         cherrypy.tree.mount(root, config=appconf)
@@ -289,6 +322,21 @@ class RequestObjectTests(helper.CPWebCase):
         self.getPage('/scheme')
         self.assertBody(self.scheme)
 
+    def test_per_request_uuid4(self):
+        self.getPage('/request_uuid4')
+        first_uuid4, _, second_uuid4 = self.body.decode().partition(' ')
+        assert (
+            uuid.UUID(first_uuid4, version=4)
+            == uuid.UUID(second_uuid4, version=4)
+        )
+
+        self.getPage('/request_uuid4')
+        third_uuid4, _, _ = self.body.decode().partition(' ')
+        assert (
+            uuid.UUID(first_uuid4, version=4)
+            != uuid.UUID(third_uuid4, version=4)
+        )
+
     def testRelativeURIPathInfo(self):
         self.getPage('/pathinfo/foo/bar')
         self.assertBody('/pathinfo/foo/bar')
@@ -495,6 +543,7 @@ class RequestObjectTests(helper.CPWebCase):
                 '/paramerrors/raise_type_error',
                 '/paramerrors/raise_type_error_with_default_param?x=0',
                 '/paramerrors/raise_type_error_with_default_param?x=0&y=0',
+                '/paramerrors/raise_type_error_decorated',
         ):
             self.getPage(uri, method='GET')
             self.assertStatus(500)
@@ -677,14 +726,14 @@ class RequestObjectTests(helper.CPWebCase):
             self.getPage('/headers/ifmatch',
                          [('If-Match', ntou('=?utf-8?q?%s?=') % c)])
             # The body should be utf-8 encoded.
-            self.assertBody(ntob('\xe2\x84\xabngstr\xc3\xb6m'))
+            self.assertBody(b'\xe2\x84\xabngstr\xc3\xb6m')
             # But the Etag header should be RFC-2047 encoded (binary)
             self.assertHeader('ETag', ntou('=?utf-8?b?4oSrbmdzdHLDtm0=?='))
 
             # Test a *LONG* RFC-2047-encoded request and response header value
             self.getPage('/headers/ifmatch',
                          [('If-Match', ntou('=?utf-8?q?%s?=') % (c * 10))])
-            self.assertBody(ntob('\xe2\x84\xabngstr\xc3\xb6m') * 10)
+            self.assertBody(b'\xe2\x84\xabngstr\xc3\xb6m' * 10)
             # Note: this is different output for Python3, but it decodes fine.
             etag = self.assertHeader(
                 'ETag',
@@ -708,7 +757,8 @@ class RequestObjectTests(helper.CPWebCase):
         self.assertBody('application/json')
 
     def test_basic_HTTPMethods(self):
-        helper.webtest.methods_with_bodies = ('POST', 'PUT', 'PROPFIND')
+        helper.webtest.methods_with_bodies = ('POST', 'PUT', 'PROPFIND',
+                                              'PATCH')
 
         # Test that all defined HTTP methods work.
         for m in defined_http_methods:
@@ -719,10 +769,51 @@ class RequestObjectTests(helper.CPWebCase):
                 self.assertBody('')
             elif m == 'TRACE':
                 # Some HTTP servers (like modpy) have their own TRACE support
-                self.assertEqual(self.body[:5], ntob('TRACE'))
+                self.assertEqual(self.body[:5], b'TRACE')
             else:
                 self.assertBody(m)
 
+        # test of PATCH requests
+        # Request a PATCH method with a form-urlencoded body
+        self.getPage('/method/parameterized', method='PATCH',
+                     body='data=on+top+of+other+things')
+        self.assertBody('on top of other things')
+
+        # Request a PATCH method with a file body
+        b = 'one thing on top of another'
+        h = [('Content-Type', 'text/plain'),
+             ('Content-Length', str(len(b)))]
+        self.getPage('/method/request_body', headers=h, method='PATCH', body=b)
+        self.assertStatus(200)
+        self.assertBody(b)
+
+        # Request a PATCH method with a file body but no Content-Type.
+        # See https://github.com/cherrypy/cherrypy/issues/790.
+        b = b'one thing on top of another'
+        self.persistent = True
+        try:
+            conn = self.HTTP_CONN
+            conn.putrequest('PATCH', '/method/request_body', skip_host=True)
+            conn.putheader('Host', self.HOST)
+            conn.putheader('Content-Length', str(len(b)))
+            conn.endheaders()
+            conn.send(b)
+            response = conn.response_class(conn.sock, method='PATCH')
+            response.begin()
+            self.assertEqual(response.status, 200)
+            self.body = response.read()
+            self.assertBody(b)
+        finally:
+            self.persistent = False
+
+        # Request a PATCH method with no body whatsoever (not an empty one).
+        # See https://github.com/cherrypy/cherrypy/issues/650.
+        # Provide a C-T or webtest will provide one (and a C-L) for us.
+        h = [('Content-Type', 'text/plain')]
+        self.getPage('/method/reachable', headers=h, method='PATCH')
+        self.assertStatus(411)
+
+        # HTTP PUT tests
         # Request a PUT method with a form-urlencoded body
         self.getPage('/method/parameterized', method='PUT',
                      body='data=on+top+of+other+things')
@@ -738,7 +829,7 @@ class RequestObjectTests(helper.CPWebCase):
 
         # Request a PUT method with a file body but no Content-Type.
         # See https://github.com/cherrypy/cherrypy/issues/790.
-        b = ntob('one thing on top of another')
+        b = b'one thing on top of another'
         self.persistent = True
         try:
             conn = self.HTTP_CONN
@@ -793,15 +884,47 @@ class RequestObjectTests(helper.CPWebCase):
         self.assertStatus(200)
 
     def test_CONNECT_method(self):
-        if getattr(cherrypy.server, 'using_apache', False):
-            return self.skip('skipped due to known Apache differences... ')
+        self.persistent = True
+        try:
+            conn = self.HTTP_CONN
+            conn.request('CONNECT', 'created.example.com:3128')
+            response = conn.response_class(conn.sock, method='CONNECT')
+            response.begin()
+            self.assertEqual(response.status, 204)
+        finally:
+            self.persistent = False
 
-        self.getPage('/method/', method='CONNECT')
-        self.assertBody('CONNECT')
+        self.persistent = True
+        try:
+            conn = self.HTTP_CONN
+            conn.request('CONNECT', 'body.example.com:3128')
+            response = conn.response_class(conn.sock, method='CONNECT')
+            response.begin()
+            self.assertEqual(response.status, 200)
+            self.body = response.read()
+            self.assertBody(b'CONNECTed to /body.example.com:3128')
+        finally:
+            self.persistent = False
+
+    def test_CONNECT_method_invalid_authority(self):
+        for request_target in ['example.com', 'http://example.com:33',
+                               '/path/', 'path/', '/?q=f', '#f']:
+            self.persistent = True
+            try:
+                conn = self.HTTP_CONN
+                conn.request('CONNECT', request_target)
+                response = conn.response_class(conn.sock, method='CONNECT')
+                response.begin()
+                self.assertEqual(response.status, 400)
+                self.body = response.read()
+                self.assertBody(b'Invalid path in Request-URI: request-target '
+                                b'must match authority-form.')
+            finally:
+                self.persistent = False
 
     def testEmptyThreadlocals(self):
         results = []
         for x in range(20):
             self.getPage('/threadlocal/')
             results.append(self.body)
-        self.assertEqual(results, [ntob('None')] * 20)
+        self.assertEqual(results, [b'None'] * 20)
